package org.openxma.dsl.generator.impl;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.mwe.core.monitor.ProgressMonitor;
import org.eclipse.xtend.XtendFacade;
import org.eclipse.xtend.expression.ExecutionContextImpl;
import org.eclipse.xtend.typesystem.emf.EmfRegistryMetaModel;
import org.openxma.dsl.common.DslConstants;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.pom.model.Component;

import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.util.ValidationError;

import com.google.common.collect.Iterables;

public class PomGenerator extends AbstractGenerator {
    private File projectDirectory;
    private String sourceFolder;
    private String resourceFolder;
    private String guiDesignProperties;

    public void init(GeneratorConfiguration generatorConfiguration) {
        super.init(generatorConfiguration);
        sourceFolder = getSourceFolderPath();
        resourceFolder = getResourceFolderPath();
        guiDesignProperties = generatorConfiguration.getProperty(GeneratorConstants.GUIDESIGNER_PROPERTIES_FILE);
    }

    private String getSourceFolderPath() {
        if (sourceFolder == null) {
            sourceFolder = toRelativePath(getConfiguration().getSourceFolder());
        }
        return sourceFolder;
    }

    private String getResourceFolderPath() {
        if (resourceFolder == null) {
            resourceFolder = toRelativePath(getConfiguration().getResourceFolder());
        }
        return resourceFolder;
    }

    private File getProjectDirectoryFile() {
        if (projectDirectory == null) {
            if (getConfiguration().getProjectFolder().isPlatformResource()) {
                IProject project = EcorePlugin.getWorkspaceRoot().getProject(
                        getConfiguration().getProjectFolder().lastSegment());
                projectDirectory = new File(project.getLocationURI());
            } else {
                projectDirectory = new File(getConfiguration().getProjectFolder().toFileString());
            }
        }
        return projectDirectory;

    }

    public void generate(Resource resource, ProgressMonitor progressMonitor) {
        if (resource == null) {
            throw new IllegalArgumentException("parameter '" + resource + "' must not be null");
        }
        if (resource.getEObject("/") instanceof Model) {
            ExecutionContextImpl executionContextImpl = new ExecutionContextImpl();
            executionContextImpl.registerMetaModel(new EmfRegistryMetaModel());
            XtendFacade xtendFacade = XtendFacade.create(executionContextImpl,
                    GeneratorConstants.EXTENSIONS_XMA_COMPONENT);
            Model model = (Model) resource.getEObject("/");
            logComponentAndLayoutStrategy(model);
            progressMonitor.subTask("extend XMA model");
            XMAComponent extendedXmaComponent = (XMAComponent) xtendFacade.call("extendXMAComponent", model);
            // It is possible that there is no XMAComponent referenced from the presentation model
            // (e.g. when only styles are defined in a presentation model)
            // -> therefore the following check for the XMAComponent
            if (extendedXmaComponent != null) {
                logger.info("Design model '" + extendedXmaComponent.eResource().getURI() + "' updated");
                try {
                    progressMonitor.subTask("validate XMA model");
                    @SuppressWarnings("unchecked")
                    List<ValidationError> validationErrors = extendedXmaComponent.validate(getProjectDirectoryFile(),
                            guiDesignProperties);
                    if (!validationErrors.isEmpty()) {
                        StringBuilder buffer = new StringBuilder();
                        logger.info("Validation error(s) in design model. Please check 'Problems View' for details.");
                        for (ValidationError validationError : validationErrors) {
                            logger.error(validationError.getMessage());                            
                            buffer.append(validationError.getObject()).append("\n").append(validationError.getMessage()).append("\n\n");
                        }
                        logger.info("Canceled.");
                        throw new RuntimeException("Validation error(s) in design model:\n" + buffer.toString());
                        
                    }
                    progressMonitor.subTask("save XMA model");
                    save(model, extendedXmaComponent);
                    progressMonitor.subTask("generate java code from XMA model");
                    extendedXmaComponent.generate(getProjectDirectoryFile(), sourceFolder, resourceFolder,
                            guiDesignProperties);
                } catch (IOException ioException) {
                    throw new WrappedException(ioException);
                }
                logger.info("Finished.");
            }
            progressMonitor.done();
        }
    }

    private void save(Model model, XMAComponent xmaComponent) {
        URI xmaResourceUri = null;
        if (xmaComponent.eResource() != null) {
            xmaResourceUri = xmaComponent.eResource().getURI();
            xmaComponent = (XMAComponent) EcoreUtil.copy(xmaComponent);
        } else {
            xmaResourceUri = model.eResource().getURI().trimFileExtension().appendFileExtension(
                    DslConstants.OPENXMA_XMA_FILE_EXTENSION);
        }
        ResourceSet xmaResourceSet = new ResourceSetImpl();
        Resource xmaResource = xmaResourceSet.createResource(xmaResourceUri);
        xmaResource.getContents().add(xmaComponent);
        try {
            xmaResource.save(Collections.emptyMap());
        } catch (IOException ioException) {
            throw new WrappedException(ioException);
        }
    }

    private void logComponentAndLayoutStrategy(Model model) {
        Iterator<Component> it = Iterables.filter(model.getElements(), Component.class).iterator();
        if (it == null || !it.hasNext())
            return;
        Component component = it.next();
        logger.info("Component: " + component.getXmaComponent().getNamClass());
        logger.info("Layout strategy: " + GeneratorConfiguration.INSTANCE.getLayoutStrategy().getClass().getName());
    }

}
